<?php

namespace EventPortal\Domain\Response\Models;

use EventPortal\Concerns\HasSchemalessAttributes;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Response extends Model
{
    use HasSchemalessAttributes;

    /** @var array<string> */
    protected $casts = [
        'data' => 'array',
    ];

    /** @var array<string> */
    protected $guarded = [];

    /**
     * @param mixed $value
     */
    public function setValueAttribute($value): void
    {
        $this->data->set('value', $value);
    }

    /**
     * @return mixed
     */
    public function getValueAttribute()
    {
        return $this->data->get('value');
    }

    public function scopeHasValue(Builder $query, string $value): Builder
    {
        return $query->where('data->value->value', $value);
    }

    public function scopeLikeValue(Builder $query, string $value): Builder
    {
        return $query->where(DB::raw('lower(data->"$.value")'), 'LIKE', '%' . strtolower($value). '%');
    }
}
