<?php

namespace EventPortal\Domain\Util;

use EventPortal\Exceptions\NoDefaultConfig;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;

class DefaultFields
{
    /** @var \Illuminate\Support\Stringable */
    private $model;

    /** @var string */
    private $action;

    public function __construct(string $modelClass, string $action)
    {
        $this->model = $this->setModel($modelClass);
        $this->action = $action;
    }

    public static function forCreation(string $modelClass): self
    {
        return new static($modelClass, 'create');
    }

    public static function forStoring(string $modelClass): self
    {
        return new static($modelClass, 'store');
    }

    public function asCollection(): Collection
    {
        return collect($this->get());
    }

    public function get(): array
    {
        $config = config("defaults.{$this->model}.fields");

        throw_if($config === null, NoDefaultConfig::class, 'Config type fields not found.');

        return $this->rejectAnyHidden($config);
    }

    private function setModel(string $modelClass): string
    {
        return Str::of($modelClass)->afterLast('\\')->lower();
    }

    private function rejectAnyHidden(array $config): array
    {
        return collect($config)
            ->reject(function ($section) {
                return $this->rejectHidden($section);
            })
            ->map(function ($section): array {
                $section['children'] = collect(Arr::get($section, 'children', []))
                    ->reject(function ($field) {
                        return $this->rejectHidden($field);
                    })
                    ->toArray();

                return $section;
            })
            ->toArray();
    }

    private function rejectHidden(array $data): bool
    {
        if (Arr::has($data, 'hideWhen') === false) {
            return false;
        }

        $hideWhen = Arr::wrap(Arr::get($data, 'hideWhen'));

        return in_array($this->action, $hideWhen, true);
    }
}
